<?php

namespace Drupal\lametric\Service;

use Drupal\Component\Serialization\SerializationInterface;
use GuzzleHttp\ClientInterface;

/**
 * Interface LaMetricNotificationInterface.
 */
interface LaMetricNotificationInterface {

  /**
   * The push URL prefix.
   */
  const PUSH_URL_PREFIX = 'https://developer.lametric.com/api/v1/dev/widget/update/com.lametric.';

  /**
   * The API token suffix.
   */
  const API_TOKEN_SUFFIX = '==';

  /**
   * LaMetricNotification constructor.
   *
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The Guzzle HTTP client instance.
   * @param \Drupal\Component\Serialization\SerializationInterface $json_serialization
   *   The JSON serialization.
   */
  public function __construct(
    ClientInterface $http_client,
    SerializationInterface $json_serialization
  );

  /**
   * Send notification to a device in the same network.
   *
   * @param string $ip
   *   The IP address.
   * @param string $key
   *   The API key.
   * @param string $text
   *   The message text.
   * @param int|null $icon
   *   (optional) The icon ID.
   *
   * @return \Exception|int|string
   *   One of the following values:
   *   - The message ID when a notification was sent successfully and the
   *     response structure is correct.
   *   - The response content when a notification was sent successfully but the
   *     response structure is invalid.
   *   - The exception object when a notification was sent unsuccessfully.
   *
   * @see https://lametric-documentation.readthedocs.io/en/latest/guides/first-steps/first-local-notification.html#send-notification
   */
  public function sendLocally($ip, $key, $text, $icon = NULL);

  /**
   * Send notification to a device via an application.
   *
   * @param string $url
   *   The push URL suffix.
   * @param string $token
   *   The access token.
   * @param string $text
   *   The message text.
   * @param int|null $icon
   *   (optional) The icon ID.
   *
   * @return \Exception|bool
   *   TRUE if a notification was sent successfully otherwise the exception object.
   *
   * @see https://lametric-documentation.readthedocs.io/en/latest/guides/first-steps/first-lametric-indicator-app.html#push-some-data
   */
  public function sendApplication($url, $token, $text, $icon = NULL);

}
